@@ -1,5 +1,6 @@
 # TODO: token 相关的初始化可以通过 default Callable 来实现
 import hashlib
+from dataclasses import dataclass,field
 import os
 import time
 from datetime import datetime
@@ -19,6 +20,60 @@ HEYWHALE_SITE = os.getenv("HEYWHALE_HOST", "https://www.heywhale.com")
 HEYWHALE_DS_BUCKET = os.getenv("HEYWHALE_DS_BUCKET", "kesci")
 
 
+@dataclass(frozen=True)
+class DatasetFileForUpload:
+    local_path: str
+    remote_path: str
+
+
+@dataclass
+class DatasetConstructor:
+    title: str
+    short_description: str
+    data_files: Set[DatasetFileForUpload] = field(default_factory=set)
+
+    def add_file(self, source: str, destination: str):
+        """
+        将文件添加到待上传的数据集中
+
+        Args:
+            source (str): 文件源路径
+            destination (str): 文件目标路径
+
+        Returns:
+            None
+        """
+        self.data_files.add(DatasetFileForUpload(source, destination))
+
+    def add_dir(self, local_dir: str, remote_dir: str):
+        """
+        递归遍历本地目录，将文件添加到远程数据集中。
+
+        Args:
+            local_dir (str): 本地目录路径。
+            remote_dir (str): 远程数据集目录路径。
+
+        Returns:
+            None
+
+        """
+        for root, _dirs, files in os.walk(local_dir):
+            for file in files:
+                file_path = os.path.join(root, file)
+                rel_path = os.path.relpath(os.path.join(root, file), local_dir)
+                path_in_dataset = os.path.join(remote_dir, rel_path)
+                self.add_file(file_path, path_in_dataset)
+
+    def push_dataset(self) -> Dataset:
+        dataset = _create_dataset(
+            title=self.title,
+            files=list(self.data_files),
+            short_description=self.short_description,
+        )
+        if dataset is None:
+            raise ValueError("Failed to create dataset")
+        return dataset
+
 def parse_datetime(date_string: str) -> datetime:
     """
     Parse a datetime string into a datetime object.
@@ -26,10 +81,15 @@ def parse_datetime(date_string: str) -> datetime:
     :param date_string: The datetime string to parse.
     :return: A datetime object.
     """
-    return datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
+    try:
+        return datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
+    except ValueError as e:
+        raise ValueError("Invalid datetime string format") from e
 
 
 def convert_to_dataset_file(files) -> List[DatasetFile]:
+    if files is None:
+        return []
     dataset_files = [
         DatasetFile(
             file.get("_id"),
@@ -83,6 +143,9 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
         # print("files_with_subpath", files_with_subpath)
         # files.extend(files_with_subpath)
         # print("return dataset files",files)
+
+        # Files里面不是某个版本的Files
+        # 这里用 downloadUrl?Version的方式重新构造一次 files
         return Dataset(
             _id=response.json().get("_id"),
             title=response.json().get("Title"),
@@ -394,6 +457,7 @@ def _download_single_file(
     commit: str,
     cache_dir: Path,
     local_dir: Optional[str | Path] = None,
+    local_path: Optional[str | Path] = None,
 ) -> Path:
     """
     Downloads a single file from a dataset.
@@ -418,6 +482,8 @@ def _download_single_file(
     )
     if local_dir is not None:
         slk_path = Path(local_dir).joinpath(filename)
+    if local_path is not None:
+        slk_path = Path(local_path)
     if slk_path.exists():
         return slk_path
     blob_path = Path(cache_dir).expanduser().joinpath("blobs").joinpath(etag)
@@ -722,12 +788,35 @@ def create_commit(
 
 def create_dataset(
     title: str,
+    local_dir: str,
+    remote_dir: Union[str | None],
+    short_description: Union[str | None],
+) -> Optional[Dataset]:
+    if short_description is None:
+        short_description = title
+    dataset_constructor = DatasetConstructor(
+        title=title, short_description=short_description, data_files=set()
+    )
+    if remote_dir is None:
+        remote_dir = ""
+    dataset_constructor.add_dir(local_dir, remote_dir)
+    
+    return _create_dataset(
+        title=dataset_constructor.title,
+        files=list(dataset_constructor.data_files),
+        short_description=dataset_constructor.short_description,
+    )
+
+
+def _create_dataset(
+    title: str,
+    files: List[DatasetFileForUpload],
     short_description: Optional[str] = None,
     folder_name: Optional[str] = None,
     token: Optional[str] = None,
     enable_download: bool = True,
 ) -> Optional[Dataset]:
-    """Creates a empty dataset
+    """Creates a dataset
 
     Args:
         title (str): The title of the dataset.
@@ -752,51 +841,42 @@ def create_dataset(
     if short_description is None:
         short_description = title
     upload_info = _get_update_token(token)
+    dataset_files: List[DatasetFile] = list()
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
-    session = boto3.Session(
-        aws_access_key_id=upload_info.ak,
-        aws_secret_access_key=upload_info.sk,
-        aws_session_token=upload_info.token,
-        region_name=upload_info.region,
-    )
-    s3 = session.client("s3", endpoint_url=upload_info.endpoint)
-    bucket_name = upload_info.bucket
-    object_key = os.path.join(
-        upload_info.prefix_to_save, "I_am_a_fake_key_to_pass_the_validation"
-    )
-    # 不能创建空文件的数据集，所以上传一个占位文件来绕过检查。
-    s3.put_object(
-        Bucket=bucket_name,
-        Key=object_key,
-        Body="I_am_a_fake_key_to_pass_the_validation",
-    )
-
+    for file in files:
+        object_key = _upload_file(file.local_path, file.remote_path, upload_info)
+        sub_path = _norm_path(str(Path(file.remote_path).parent))
+        dataset_files.append(
+            DatasetFile(_id="", size=0, key=object_key, sub_path=sub_path)
+        )
     data = {
         "Title": title,
         "ShortDescription": short_description,
         "FolderName": folder_name,
         "EnableDownload": enable_download,
         "Type": 0,
-        "Files": [object_key],  # 没用FileStructure，因为本来也是个假文件
+        "Files": [file.key for file in dataset_files],
+        "FilesStructure": [
+            {"Token": file.key, "SubPath": file.sub_path} for file in dataset_files
+        ],
     }
-
+    #print(f"data {data}")
     response = requests.post(
         url,
         json=data,
         headers=headers,
         timeout=10,
     )
-
     if response.status_code == 200:
         document = response.json().get("document")
-        files = convert_to_dataset_file(document.get("Files"))
         return Dataset(
             _id=document.get("_id"),
             title=document.get("Title"),
             short_description=document.get("ShortDescription"),
             folder_name=document.get("FolderName"),
-            files=files,  # empty actually
+            files=dataset_files,
+            commits=[],
             created_at=parse_datetime(document.get("CreateDate")),
             updated_at=parse_datetime(document.get("UpdateDate")),
         )
@@ -857,6 +937,7 @@ def list_datasets(
         headers=headers,
         params={
             "perPage": limit,
+            "page": 0,
             "Title": title,
             "startDate": start_date,
             "endDate": end_date,
@@ -869,10 +950,21 @@ def list_datasets(
         for d in document.get("data"):
             files = convert_to_dataset_file(d.get("Files"))
             files_with_subpath = convert_to_dataset_file(d.get("FilesStructure"))
-            files_key_set = set([file.key for file in files])
-            for file in files:
-                if file.key not in files_key_set:
-                    files_with_subpath.append(file)
+            if len(files) > 0:
+                files_key_set = set([file.key for file in files])
+                for file in files:
+                    if file.key not in files_key_set:
+                        files_with_subpath.append(file)
+            commits = []
+            if d.get("DatasetVersions") is not None:
+                commits = [
+                    DatasetCommit(
+                        commit.get("_id"),
+                        commit.get("CommitMessage"),
+                        parse_datetime(commit.get("CreateDate")),
+                    )
+                    for commit in d.get("DatasetVersions")
+                ]
             datasets.append(
                 Dataset(
                     _id=d.get("_id"),
@@ -880,6 +972,7 @@ def list_datasets(
                     short_description=d.get("ShortDescription"),
                     folder_name=d.get("FolderName"),
                     files=files_with_subpath,
+                    commits=commits,
                     created_at=parse_datetime(d.get("CreateDate")),
                     updated_at=parse_datetime(d.get("UpdateDate")),
                 )
