@@ -16,9 +16,10 @@ class TestDownloadFile(unittest.TestCase):
             print(f"An error occurred: {err}")
 
     def test_download(self):
-        dataset_id_tmp = "66b08ec9898e74a8232bb2d1"
+        dataset_id_tmp = "64254cbfb9d501d04418c291"
         try:  # Example assertion
-            download_file(dataset_id_tmp, "b_local_dir/heart_2020_cleaned.csv")
+            path = download_file(dataset_id_tmp, "README.md")
+            print(path)
         except Exception as err:
             print(f"An error occurred: {err}")
 
